// Copyright © 2022-2025 Gerald B. Cox
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package common

import (
    "fmt"
    "log/slog"
    "sync"
    "time"
)

var (
    recentScrobbles = make(map[string]int64)
    mu              sync.Mutex
    dedupWindow     = 300 // seconds (5 minutes)
)

// IsDuplicateScrobble returns true if the same user+track+session was scrobbled
// within the deduplication window. Otherwise, it records the scrobble and returns false.
func IsDuplicateScrobble(user, trackID, sessionID string) bool {
    if trackID == "" || sessionID == "" {
        slog.Debug("Deduplication skipped: missing trackID or sessionID", "user", user, "trackID", trackID, "sessionID", sessionID)
        return false
    }

    now := time.Now().Unix()
    key := fmt.Sprintf("%s:%s:%s", user, trackID, sessionID)

    mu.Lock()
    defer mu.Unlock()

    ts, ok := recentScrobbles[key]
    withinWindow := ok && now-ts < int64(dedupWindow)

    slog.Debug("Deduplication check",
        "user", user,
        "trackID", trackID,
        "sessionID", sessionID,
        "key", key,
        "now", now,
        "last_seen", ts,
        "within_window", withinWindow,
    )

    if withinWindow {
        return true
    }

    recentScrobbles[key] = now
    return false
}
